/*

The following procedures are used to format a string/list/set of items after the standard syntax that SQL insert, select and update use.


SQLFormat_Insert(_tbl, list/L)|
*******************************
ARGUMENTS:

_tbl 	= The name of the table being inserted into
L		= The list of items to insert

Note that L is expected to be associative (Associative value = Value to associate with variable on insert.

RETURNS:

A string in the following syntax:
	"INSERT INTO `[_tbl]` (`item_1`,`item_2`,`item_3`) VALUES ('value_1','value_2','value_3')"
********************************

SQLFormat_Select(_tbl, what = "*", cond = "WHERE 1")|
*****************************************************
ARGUMENTS:

_tbl	= The name of the table being selected from
what	= What variables to select from the table match.. Everything(*) by default.
cond	= The condition to match entries by. Every entry(WHERE 1) by default.

Note that what can also be a list of items, to select multiple variables from the table match.

RETURNS:

A string in the following syntax:
	If what is a single item...
		"SELECT [what] FROM `[_tbl]` [cond]"
	If what is a list of items...
		"SELECT [what[1]],[what[2]],[what[3]],... FROM `[_tbl]` [cond]"

Note that cond may not be a list, and is entered exactly as the text string you supply for that argument.
*****************************************************

SQLFormat_Update(_tbl, list/what, cond = "WHERE 1")|
****************************************************
ARGUMENTS:

_tbl	= The name of the table being updated to
what	= The list of variable(s) to update in the table
cond	= The condition to match what rows to update. All rows by default (WHERE 1)

Note that what is expected to be an associative list, where the associative value is the value you want updated under that variable name

RETURNS:

A string in the following syntax:
	"UPDATE `[_tbl]` SET `[what[1]]` = '[what[1]] associative value',`[what[2]]` = '[what[2]] associative value',... [cond]"
****************************************************

*/
proc
	SQLFormat_Cond(list/L)
		if(!L) return 0
		if(!istype(L, /list)) L = list(L)

		. = "WHERE "
		for(var/i = 1, i <= length(L), i++)
			var/j = L[i]
			if(isnull(L[j])) // No associative value
				. += "[L[i]][i == length(L) ? "":" AND "]"
			else
				var/v = L[j]
				. += "[j] = [istext(v) ? "'[v]'":"[v]"][i == length(L) ? "":" AND "]"

	SQLFormat_Insert(_tbl, list/L)
		if(!L || !istype(L, /list))
			return 0

		var/i = 1
		var/list_len = length(L)

		. = "INSERT INTO `[_tbl]` ("
		for(i, i <= list_len, i++)
			. += "`[L[i]]`[i == list_len ? "":","]"
		. += ") VALUES ("
		i = 1
		for(i, i <= list_len, i++)
			var/j = L[i]
			var/value = L[j]
			. += "'[value]'[i == list_len ? "":","]"
		. += ")"

	SQLFormat_Select(_tbl, what = "*", cond = "WHERE 1", limit = "")
		if(istext(what))
			. = "SELECT [what] FROM `[_tbl]` [cond][limit ? " LIMIT [limit]":""]"
		else if(istype(what, /list))
			. = "SELECT "
			var/list/L = what
			var/list_len = length(L)
			for(var/i = 1, i <= list_len, i++)
				var/v = L[i]
				. += "[v][i == list_len ? "":","]"
			. += " FROM [_tbl] [cond][limit ? " LIMIT [limit]":""]"

	SQLFormat_Update(_tbl, list/what, cond = "WHERE 1")
		if(istype(what, /list))
			. = "UPDATE `[_tbl]` SET "
			var/list_len = length(what)
			for(var/i = 1, i <= list_len, i++)
				var/v = what[i]
				. += "`[v]` = '[what[v]]'[i == list_len ? "":","]"
			. += " [cond]"